@@ -1,6 +1,6 @@
 /* sogoWebDAV.js - This file is part of "SOGo Connector", a Thunderbird extension.
  *
- * Copyright: Inverse inc., 2006-2014
+ * Copyright: Inverse inc., 2006-2016
  *
  * "SOGo Connector" is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published by
@@ -204,7 +204,17 @@ sogoWebDAV.prototype = {
 
         /* If set too early, the method can change to "PUT" when initially set to "PROPFIND"... */
         httpChannel.requestMethod = method;
+	/* PARCHE SARENET*/
+        if (method == "PUT")
+        {
+            this.synchronous = true;
+        }
 
+        if (method == "DELETE")
+        {
+            this.synchronous = true;
+        }
+        /* PARCHE SARENET*/
         if (this.synchronous) {
             let inStream = httpChannel.open();
             let byteStream = Components.classes["@mozilla.org/binaryinputstream;1"]
@@ -253,6 +263,9 @@ sogoWebDAV.prototype = {
             if (status == 499) {
                 dump("xmlRequest: received status 499 for url: " + this.url + "\n");
             }
+	    else if (status == 412) {
+		dump("xmlRequest: received status 412 - precondition failed for url: " + this.url + "\n");
+	    }
             else {
                 let visitor = {};
                 visitor.visitHeader = function(aHeader, aValue) {
@@ -304,7 +317,7 @@ sogoWebDAV.prototype = {
         }
         catch(e) {
             dump("sogoWebDAV.js: an exception occured\n" + e + "\n"
-                 + e.fileName + ":" + e.lineNumber + "\n");
+                 + e.fileName + ":" + e.lineNumber + "\n\nstack: " + e.stack);
             let uri = aChannel.URI;
             if (uri) {
                 dump("url: " + uri.spec + "\n");
@@ -342,7 +355,8 @@ sogoWebDAV.prototype = {
                         dump("NOTICE: uploading modified vcard without etag\n");
 	                this._sendHTTPRequest(operation,
                                               parameters.data,
-                                              { "content-type": parameters.contentType });
+					      { "content-type": parameters.contentType,
+				              "If-None-Match": "*" });
                     }
                 }
 	    }
@@ -399,7 +413,7 @@ sogoWebDAV.prototype = {
         let nsDict = { "DAV:": "D" };
         let propPart = "";
         let nsCount = 0;
-        for each (let prop in props) {
+        for (let prop of props) {
             let propParts = prop.split(" ");
             let ns = propParts[0];
             let nsS = nsDict[ns];
