@@ -2,7 +2,13 @@ window.addEventListener("load", SCOnLoad, false);
 
 function SCOnLoad()
 {
-    let item = window.calendarItem;
+    let item = window.calendarEvent;
+
+    if (item === null) {
+	// TODO: handle Thunderbird 52 with iframe options
+	return;
+    }
+
     if (item.id === null) { /* item is new */
         let prefName = null;
         if (cal.isEvent(item)) {
