@@ -1,6 +1,6 @@
 /* sync.addressbook.groupdav.js - This file is part of "SOGo Connector", a Thunderbird extension.
  *
- * Copyright: Inverse inc., 2006-2014
+ * Copyright: Inverse inc., 2006-2016
  *     Email: support@inverse.ca
  *       URL: http://inverse.ca
  *
@@ -251,6 +251,7 @@ GroupDavSynchronizer.prototype = {
                     // this.dumpCard(card);
                     dump("  new card '" + card.displayName + "' will be uploaded\n");
                     key = new UUID() + ".vcf";
+                    card.setProperty(kNameKey, key);
                     this.localCardUploads[key] = card;
                     uploads++;
                 }
@@ -385,7 +386,7 @@ GroupDavSynchronizer.prototype = {
         else if (data.query == "server-sync-query")
             this.onServerSyncQueryComplete(status, response);
         else if (data.query == "card-upload")
-            this.onCardUploadComplete(status, response, data.key, data.data, headers);
+	    this.onCardUploadComplete(status, response, data.key, data.data, headers);
         else if (data.query == "list-upload")
             this.onListUploadComplete(status, response, data.key, data.data, headers);
         else if (data.query == "server-finalize-propfind")
@@ -411,10 +412,11 @@ GroupDavSynchronizer.prototype = {
     onCardDownloadComplete: function(status, data, key) {
         this.remainingDownloads--;
         this.progressMgr.updateAddressBook(this.gURL);
+        let pos;
         if (Components.isSuccessCode(status)
             && data
-            && (data.toLowerCase().indexOf("begin:vcard") == 0))
-            this.importCard(key, data);
+            && ((pos = data.toLowerCase().indexOf("begin:vcard")) >= 0))
+            this.importCard(key, data.substr(pos));
         else
             this.appendFailure(status, key);
 
@@ -427,10 +429,11 @@ GroupDavSynchronizer.prototype = {
     onListDownloadComplete: function(status, data, key) {
         this.remainingDownloads--;
         this.progressMgr.updateAddressBook(this.gURL);
+        let pos;
         if (Components.isSuccessCode(status)
             && data
-            && (data.toLowerCase().indexOf("begin:vlist") == 0))
-            this.importList(key, data);
+            && ((pos = data.toLowerCase().indexOf("begin:vlist")) >= 0))
+            this.importList(key, data.substr(pos));
         else
             this.appendFailure(status, key);
         if (this.remainingDownloads == 0) {
@@ -467,10 +470,10 @@ GroupDavSynchronizer.prototype = {
             onDAVQueryComplete: function(status, response, headers, data) {
                 if (status > 199 && status < 400) {
                     let responses = response["multistatus"][0]["response"];
-                    for each (let response in responses) {
+                    for (let response of responses) {
                         let href = response["href"][0];
                         let propstats = response["propstat"];
-                        for each (let propstat in propstats) {
+                        for (let propstat of propstats) {
                             if (propstat["status"][0].indexOf("HTTP/1.1 200") == 0) {
                                 let prop = propstat["prop"][0];
                                 if (prop["getetag"] && prop["getetag"].length > 0) {
@@ -514,12 +517,23 @@ GroupDavSynchronizer.prototype = {
             let console = Components.classes["@mozilla.org/consoleservice;1"]
                 .getService(Components.interfaces.nsIConsoleService);
 
-            this.appendFailure(status, card);
+	    if (status == 412) {
+		console.logStringMessage("Precondition failed for card: " + cardURL
+					 + ".\nHTTP Status Code:" + status + "\nRedownloading!");
+		let data = {query: "vcard-download", data: key};
+		let itemDict = { etag: headers["etag"], type: "text/vcard" };
+		this.serverDownloads[key] = itemDict;
+		this.serverDownloadsCount++;
+                this.remainingDownloads++;
+                let request = new sogoWebDAV(cardURL, this, data);
+                request.get("text/vcard");
+	    }
+	    else {
+		this.appendFailure(status, card);
+		console.logStringMessage("Upload failure uploading card: " + cardURL
+					 + ".\nHTTP Status Code:" + status + "\n" + this.cardToString(card));
+	    }
             this.localUploads--;
-
-
-            console.logStringMessage("Upload failure uploading card: " + cardURL
-                                     + ".\nHTTP Status Code:" + status + "\n" + this.cardToString(card));
         }
 
         this.progressMgr.updateAddressBook(this.gURL);
@@ -600,38 +614,39 @@ GroupDavSynchronizer.prototype = {
                 else
                 {
                     // List of all card properties which may be deleted e.g., via web interface
-                    deleteableProp = ['HomeAddress'        ,
-                                      'WorkCity'           ,
-                                      'FaxNumber'          ,
-                                      'Company'            ,
-                                      'HomeAddress2'       ,
-                                      'HomeCity'           ,
-                                      'WorkCountry'        ,
-                                      'WorkZipCode'        ,
-                                      'HomeCountry'        ,
-                                      'BirthYear'          ,
-                                      'CellularNumber'     ,
-                                      'FirstName'          ,
-                                      'Notes'              ,
-                                      'WorkState'          ,
-                                      'LastName'           ,
-                                      'HomeState'          ,
-                                      'PrimaryEmail'       ,
-                                      'BirthDay'           ,
-                                      'WebPage2'           ,
-                                      'WorkAddress2'       ,
-                                      'BirthMonth'         ,
-                                      'Categories'         ,
-                                      'NickName'           ,
-                                      'WorkAddress'        ,
-                                      'HomeZipCode',
-                                      'WebPage1',
-                                      'WorkPhone',
-                                      '_AimScreenName',
-                                      'PagerNumber',
-                                      'SecondEmail',
-                                      'HomePhone'
-                                      ];
+                    let deleteableProp = [
+                        'HomeAddress'        ,
+                        'WorkCity'           ,
+                        'FaxNumber'          ,
+                        'Company'            ,
+                        'HomeAddress2'       ,
+                        'HomeCity'           ,
+                        'WorkCountry'        ,
+                        'WorkZipCode'        ,
+                        'HomeCountry'        ,
+                        'BirthYear'          ,
+                        'CellularNumber'     ,
+                        'FirstName'          ,
+                        'Notes'              ,
+                        'WorkState'          ,
+                        'LastName'           ,
+                        'HomeState'          ,
+                        'PrimaryEmail'       ,
+                        'BirthDay'           ,
+                        'WebPage2'           ,
+                        'WorkAddress2'       ,
+                        'BirthMonth'         ,
+                        'Categories'         ,
+                        'NickName'           ,
+                        'WorkAddress'        ,
+                        'HomeZipCode',
+                        'WebPage1',
+                        'WorkPhone',
+                        '_AimScreenName',
+                        'PagerNumber',
+                        'SecondEmail',
+                        'HomePhone'
+                        ];
                     if (deleteableProp.indexOf(propName) == -1) {
                         dump("  Property "+propName+" is not deletable. Ignore.\n");
                     }
@@ -641,7 +656,7 @@ GroupDavSynchronizer.prototype = {
                            search for all properties in new card if current propName is still available.
                            If not, remove propName from oldCard. */
                         let allNewCardProperties = card.properties;
-                        propertyStillAvailable = false;
+                        let propertyStillAvailable = false;
                         while (allNewCardProperties.hasMoreElements()) {
                             let propNew = allNewCardProperties.getNext().QueryInterface(Components.interfaces.nsIProperty);
                             let propNameNew = String(propNew.name);
@@ -670,6 +685,7 @@ GroupDavSynchronizer.prototype = {
             dump("  new card\n");
             this.gAddressBook.addCard(card);
             this.localCardPointerHash[key] = card;
+            this.localCardVersionHash[key] = card.getProperty(kETagKey, "-1");
         }
     },
     importList: function(key, data) {
@@ -863,10 +879,10 @@ GroupDavSynchronizer.prototype = {
 
         if (status > 199 && status < 400 && jsonResponse) {
             let responses = jsonResponse["multistatus"][0]["response"];
-            for each (let response in responses) {
+            for (let response of responses) {
                 let href = response["href"][0];
                 let propstats = response["propstat"];
-                for each (let propstat in propstats) {
+                for (let propstat of propstats) {
                     if (propstat["status"][0].indexOf("HTTP/1.1 200") == 0) {
                         if (href[href.length-1] != '/')
                             href += '/';
@@ -982,10 +998,10 @@ GroupDavSynchronizer.prototype = {
         if (jsonResponse) {
             if (status > 199 && status < 400) {
                 let responses = jsonResponse["multistatus"][0]["response"];
-                for each (let response in responses) {
+                for (let response of responses) {
                     let href = response["href"][0];
                     let propstats = response["propstat"];
-                    for each (let propstat in propstats) {
+                    for (let propstat of propstats) {
                         if (propstat["status"][0].indexOf("HTTP/1.1 200") == 0) {
                             let prop = propstat["prop"][0];
                             if (href != this.gURL) {
@@ -1205,7 +1221,7 @@ new:
                     = jsonResponse["multistatus"][0]["sync-token"][0];
                 let responses = jsonResponse["multistatus"][0]["sync-response"];
                 if (responses) { /* old webdav sync */
-                    for each (let response in responses) {
+                    for (let response of responses) {
                         let href = response["href"][0];
                         let keyArray = href.split("/");
                         let key = keyArray[keyArray.length - 1];
@@ -1213,7 +1229,7 @@ new:
                         let itemStatus = response["status"][0].substr(9, 3);
                         if (itemStatus == "200" || itemStatus == "201") {
                             let propstats = response["propstat"];
-                            for each (let propstat in propstats) {
+                            for (let propstat of propstats) {
                                 let propStatus = propstat["status"][0].substr(9, 3);
                                 if (propStatus == "200" && href != this.gURL) {
                                     handleAddOrModify(key, itemStatus, propstat);
@@ -1229,14 +1245,14 @@ new:
                 }
                 else { /* new webdav sync */
                     responses = jsonResponse["multistatus"][0]["response"];
-                    for each (let response in responses) {
+                    for (let response of responses) {
                         let href = response["href"][0];
                         let keyArray = href.split("/");
                         let key = keyArray[keyArray.length - 1];
 
                         let propstats = response["propstat"];
                         if (propstats) {
-                            for each (let propstat in propstats) {
+                            for (let propstat of propstats) {
                                 let statusTag = propstat["status"];
                                 let itemStatus  = statusTag[0].substr(9, 3);
                                 if ((itemStatus == "200"
@@ -1420,7 +1436,7 @@ new:
 
     processCardDeletes: function() {
         let deletes = [];
-        for each (let key in this.serverDeletes) {
+        for (let key of this.serverDeletes) {
             if (this.localCardPointerHash[key])
                 deletes.push(key);
         }
@@ -1446,7 +1462,7 @@ new:
         // 		let deleteListStringForTestPurposes = "";
         //Filling the Server deleted cards Hash
 
-        for each (let key in this.serverDeletes) {
+        for (let key of this.serverDeletes) {
             let listCard = this.localListPointerHash[key];
             if (listCard) {
                 let attributes = new GroupDAVListAttributes(listCard.mailListURI);
@@ -1488,11 +1504,11 @@ new:
     onServerFinalizeComplete: function(status, jsonResponse) {
         if (status > 199 && status < 400) {
             let responses = jsonResponse["multistatus"][0]["response"];
-            for each (let response in responses) {
+            for (let response of responses) {
                 let href = response["href"][0];
                 //  				dump("href: " + href + "\n");
                 let propstats = response["propstat"];
-                for each (let propstat in propstats) {
+                for (let propstat of propstats) {
                     if (propstat["status"][0].indexOf("HTTP/1.1 200") == 0) {
                         if (href[href.length-1] != '/')
                             href += '/';
@@ -1734,9 +1750,9 @@ function GetSyncNotifyGroupdavAddressbook(uri, ab, origin) {
                 if (cbData.synchronizer.serverDownloadsCount > 1)
                     texte += cbData.synchronizer.serverDownloadsCount+cbData.notificationsStrings['notificationsDownloads'];
 
-                if (cbData.synchronizer.serverDeletes <= 1)
+                if (cbData.synchronizer.serverDeletes.length <= 1)
                     texte += cbData.synchronizer.serverDeletes.length+cbData.notificationsStrings['notificationsDelete'];
-                if (cbData.synchronizer.serverDeletes > 1)
+                if (cbData.synchronizer.serverDeletes.length > 1)
                     texte += cbData.synchronizer.serverDeletes.length+cbData.notificationsStrings['notificationsDeletes'];
                
             } else {
