@@ -1,7 +1,6 @@
 /* vcards.utils.js - This file is part of "SOGo Connector", a Thunderbird extension.
  *
- * Copyright: Inverse inc., 2006-2010
- *    Author: Robert Bolduc, Wolfgang Sourdeau
+ * Copyright: Inverse inc., 2006-2016
  *     Email: support@inverse.ca
  *       URL: http://inverse.ca
  *
@@ -255,7 +254,7 @@ function CreateCardFromVCF(vcard) {
             version = vcard[i]["values"][0];
         }
     }
-    if (version[0] == "3")
+    if (version[0] >= "3")
         defaultCharset = "utf-8";
 
     for (let i = 0; i < vcard.length; i++) {
@@ -384,12 +383,7 @@ let _insertCardMethods = {
             props["SecondEmail"] = values[0];
         }
         else {
-            if (props["PrimaryEmail"] && props["PrimaryEmail"].length > 0) {
-                props["SecondEmail"] = values[0];
-            }
-            else {
-                props["PrimaryEmail"] = values[0];
-            }
+            props["PrimaryEmail"] = values[0];
         }
     },
     url: function(props, parameters, values) {
@@ -539,7 +533,22 @@ function InsertCardData(card, tag, parameters, values) {
                 card.deleteProperty(k);
             }
             else {
-                card.setProperty(k, properties[k]);
+                let tmpPrimaryEmail = card.getProperty("PrimaryEmail", "");
+                let tmpSecondEmail = card.getProperty("SecondEmail", "");
+                if (k == "PrimaryEmail") {
+                    if (tmpPrimaryEmail.length == 0) {
+                        card.setProperty(k, properties[k]);
+                    }
+                    else if (tmpSecondEmail.length == 0) {
+                        card.setProperty("SecondEmail", properties[k]);
+                    }
+                }
+                else if (k == "SecondEmail" && tmpSecondEmail.length == 0) {
+                    card.setProperty("SecondEmail", properties[k]);
+                }
+                else {
+                    card.setProperty(k, properties[k]);
+                }
             }
             // if (k == "PhotoURI" || k == "PhotoName") {
             //     dump("  card value: " + card.getProperty(k, "(nil)") + "\n");
@@ -850,7 +859,7 @@ function deducePhotoTypeFromExt(photoName) {
     let dotParts = photoName.split(".");
     if (dotParts.length > 1) {
         let lastPart = (dotParts[dotParts.length - 1]).toUpperCase();
-        if (lastPart == "JPG") {
+        if (lastPart == "JPG" || lastPart == "JPEG") {
             type = "JPEG";
         }
         else if (lastPart == "PNG") {
@@ -981,7 +990,7 @@ function saveImportedPhoto(content, ext) {
      in the SOGo cache as a separate file which will not be destroyed by a
      further save operation */
     let lastFile = null;
-    for each (let bool in [false, true]) {
+    for (let bool of [false, true]) {
         let file = photoFileFromName(photoName, bool);
         /* 0700 is specified here because Thunderbird is too self-sufficient
          to respect the environment umask */
